/**
 * Created by IntelliJ IDEA.
 * User: Alexander
 * Date: Nov 12, 2002
 * Time: 9:42:18 PM
 */
package ru.ezhiki.automat.human.logic;

import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.*;

import java.io.FileReader;

import ru.ezhiki.utils.MathUtils;

/**
 * Loads actions from an xml-file.
 */
public class ActionsLoader extends DefaultHandler
{
    /**
     * Storage of loaded actions.
     */
    private Actions actions = null;

    private Position currentPosition = null;
    private Action currentAction = null;
    private JointAction currentJointAction = null;

    /**
     * Look the overrided method description.
     */
    public void startElement(String uri, String localName,
                             String qName, Attributes attr)
            throws SAXException
    {
        if (localName.equals("actions"))
        {
            actions = new Actions(attr.getValue("name"), attr.getValue("authors"), attr.getValue("date"));
        }
        else if (localName.equals("position"))
        {
            currentPosition = new Position(attr.getValue("name"));
        }
        else if (localName.equals("jointAction"))
        {
            currentJointAction = new JointAction(attr.getValue("name"), attr.getValue("speedTurn"), attr.getValue("speedTranslate"));
        }
        else if (localName.equals("location"))
        {
            currentJointAction.setLocation(new Double(attr.getValue("x")).doubleValue(),
                                           new Double(attr.getValue("y")).doubleValue(),
                                           new Double(attr.getValue("z")).doubleValue());
        }
        else if (localName.equals("angles"))
        {
            currentJointAction.setAngles(MathUtils.deg2rad(new Double(attr.getValue("x")).doubleValue()),
                                         MathUtils.deg2rad(new Double(attr.getValue("y")).doubleValue()),
                                         MathUtils.deg2rad(new Double(attr.getValue("z")).doubleValue()));
        }
        else if (localName.equals("action"))
        {
            currentAction = new Action(attr.getValue("type"), attr.getValue("value"));
        }
    }

    /**
     * Look the overrided method description.
     */
    public void endElement(String uri, String localName, String qName)
            throws SAXException
    {
        if (localName.equals("actions"))
        {
        }
        else if (localName.equals("position"))
        {
            actions.addPosition(currentPosition);
        }
        else if (localName.equals("jointAction"))
        {
            currentPosition.addJointAction(currentJointAction);
        }
        else if (localName.equals("action"))
        {
            actions.addAction(currentAction);
        }
    }

    /**
     * Constructor.
     */
    public ActionsLoader()
    {
    }

    /**
     * Loads the file and returns actions object.
     */
    public Actions loadFile(String fName)
    {
        try
        {
            XMLReader reader = XMLReaderFactory.createXMLReader("org.apache.crimson.parser.XMLReaderImpl");
            reader.setContentHandler(this);
            reader.parse(new InputSource(new FileReader(fName)));
        }
        catch (SAXParseException e)
        {
            System.out.println("Error at line: " + e.getLineNumber() + ", column: " + e.getColumnNumber() + ", Id: " + e.getPublicId());
            e.printStackTrace();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }

        return actions;
    }
}
