/**
 * Created by IntelliJ IDEA.
 * User: Alexander
 * Date: Nov 12, 2002
 * Time: 10:44:52 PM
 */
package ru.ezhiki.utils;

import java.util.logging.Logger;

/**
 * Class for synchronizing actions. It uses simple counter for storing number
 * of actions, now being executed. If number == 0, then no actions are currently
 * performed.
 *
 * It is made for correct actionList going. Because every action has some subactions
 * and these subactions can finish at different times.
 *
 * But i need next action to begin only after all previous action's subactions are finished.
 * That's where from this class started.
 *
 */
public class Signaller
{
    /**
     * Singleton instance.
     */
    private static Signaller signaller = new Signaller();

    /**
     * number of currently executing subactions
     */
    private Integer signalsCnt = new Integer(0);

    /**
     * Automat state.
     */
    private int y0 = 0;
    private Logger logger = null;

    /**
     * I need only one instance of signaller per
     * application, so constructor is private.
     */
    private Signaller()
    {
        logger = Logger.getLogger("ru.ezhiki.automat.human.logger");
    }

    /**
     * Getter for the singleton.
     * @return singleton instance.
     */
    public static Signaller getSignaller()
    {
        return signaller;
    }

    /**
     * Does the signaller has signals.
     * @return true, if number of signals more, than zeor, false otherwise.
     */
    public boolean hasSignals()
    {
        synchronized (signalsCnt)
        {
            if (signalsCnt.intValue() > 0)
                return true;
            else
                return false;
        }
    }

    /**
     * Put a signal.
     */
    public void addSignal()
    {
        A1(0);
    }

    /**
     * Remove a signal.
     */
    public void removeSignal()
    {
        A1(1);
    }

    /**
     * Clear signaller.
     */
    public synchronized void clear()
    {
        synchronized (signalsCnt)
        {
            signalsCnt = new Integer(0);
        }
    }

    /**
     * Standard overiding.
     * @return string, representing current number of sygnals.
     */
    public String toString()
    {
        synchronized (signalsCnt)
        {
            return signalsCnt.toString();
        }
    }

    /**
     * Signaller automat implementation.
     * @param e - event number
     */
    private void A1(int e)
    {
//        if (logger != null)
//            logger.info(">>> A1 (Signaller)");

        switch(y0)
        {
            case 0:
                if (e == 0) { z1_0();            y0 = 1; }
                else
                if (e == 1) {                            }
            break;
            case 1:
                if (e == 0) { z1_0();                    }
                else
                if (e == 1 && x1()) { z1_1();            }
                else
                if (e == 1 && !x1()) { z1_1();   y0 = 0; }
            break;
        }

//        if (logger != null)
//            logger.info("<<< A1 (Signaller)");
    }

    /**
     * Is there only one signal?
     * @return
     */
    private boolean x1()
    {
        return (signalsCnt.intValue() > 1);
    }

    /**
     * Add a signal
     */
    private void z1_0()
    {
        if (logger != null)
            logger.info(" +  A1->z1_0 ( )");

        synchronized (signalsCnt)
        {
            signalsCnt = new Integer(signalsCnt.intValue() + 1);
        }
    }

    /**
     * Remove a signal
     */
    private void z1_1()
    {
        if (logger != null)
            logger.info(" -  A1->z1_0 ( )");

        synchronized (signalsCnt)
        {
            if (signalsCnt.intValue() == 0)
                return;
            else if (signalsCnt.intValue() == 1)
                signalsCnt = new Integer(0);
            else
                signalsCnt = new Integer(signalsCnt.intValue() - 1);
        }
    }

    public void setLogger(Logger logger)
    {
        this.logger = logger;
    }
}
